@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -31,9 +31,141 @@
 **************************************************************************/
 
 #include "basetextdocumentlayout.h"
+#include <utils/qtcassert.h>
 
 using namespace TextEditor;
 
+namespace TextEditor {
+namespace Internal {
+
+class DocumentMarker : public ITextMarkable
+{
+    Q_OBJECT
+public:
+    DocumentMarker(QTextDocument *);
+    ~DocumentMarker();
+
+    TextMarks marks() const { return m_marksCache; }
+
+    // ITextMarkable
+    bool addMark(ITextMark *mark);
+    TextMarks marksAt(int line) const;
+    void removeMark(ITextMark *mark);
+    void updateMark(ITextMark *mark);
+
+    void removeMarkFromMarksCache(TextEditor::ITextMark *mark);
+private:
+    double recalculateMaxMarkWidthFactor() const;
+
+    TextMarks m_marksCache; // not owned
+    QTextDocument *document;
+};
+
+DocumentMarker::DocumentMarker(QTextDocument *doc)
+  : ITextMarkable(doc), document(doc)
+{
+}
+
+DocumentMarker::~DocumentMarker()
+{
+
+}
+
+bool DocumentMarker::addMark(TextEditor::ITextMark *mark)
+{
+    if (mark->markableInterface())
+        return false;
+    QTC_ASSERT(mark->lineNumber() >= 1, return false);
+    int blockNumber = mark->lineNumber() - 1;
+    BaseTextDocumentLayout *documentLayout =
+        qobject_cast<BaseTextDocumentLayout*>(document->documentLayout());
+    QTC_ASSERT(documentLayout, return false);
+    QTextBlock block = document->findBlockByNumber(blockNumber);
+
+    if (block.isValid()) {
+        TextBlockUserData *userData = BaseTextDocumentLayout::userData(block);
+        userData->addMark(mark);
+        m_marksCache.append(mark);
+        mark->updateLineNumber(blockNumber + 1);
+        QTC_CHECK(mark->lineNumber() == blockNumber + 1); // Checks that the base class is called
+        mark->updateBlock(block);
+        documentLayout->hasMarks = true;
+        documentLayout->maxMarkWidthFactor = qMax(mark->widthFactor(),
+            documentLayout->maxMarkWidthFactor);
+        documentLayout->requestUpdate();
+        mark->setMarkableInterface(this);
+        return true;
+    }
+    return false;
+}
+
+double DocumentMarker::recalculateMaxMarkWidthFactor() const
+{
+    double maxWidthFactor = 1.0;
+    foreach (const ITextMark *mark, marks())
+        maxWidthFactor = qMax(mark->widthFactor(), maxWidthFactor);
+    return maxWidthFactor;
+}
+
+TextEditor::TextMarks DocumentMarker::marksAt(int line) const
+{
+    QTC_ASSERT(line >= 1, return TextMarks());
+    int blockNumber = line - 1;
+    QTextBlock block = document->findBlockByNumber(blockNumber);
+
+    if (block.isValid()) {
+        if (TextBlockUserData *userData = BaseTextDocumentLayout::testUserData(block))
+            return userData->marks();
+    }
+    return TextMarks();
+}
+
+void DocumentMarker::removeMarkFromMarksCache(TextEditor::ITextMark *mark)
+{
+    BaseTextDocumentLayout *documentLayout =
+        qobject_cast<BaseTextDocumentLayout*>(document->documentLayout());
+    QTC_ASSERT(documentLayout, return)
+    bool needUpdate = m_marksCache.removeOne(mark);
+    if (m_marksCache.isEmpty()) {
+        documentLayout->hasMarks = false;
+        needUpdate = true;
+    }
+
+    if (needUpdate) {
+        documentLayout->maxMarkWidthFactor = recalculateMaxMarkWidthFactor();
+        updateMark(0);
+    }
+}
+
+void DocumentMarker::removeMark(TextEditor::ITextMark *mark)
+{
+    BaseTextDocumentLayout *documentLayout =
+        qobject_cast<BaseTextDocumentLayout*>(document->documentLayout());
+    QTC_ASSERT(documentLayout, return)
+
+    QTextBlock block = document->begin();
+    while (block.isValid()) {
+        if (TextBlockUserData *data = static_cast<TextBlockUserData *>(block.userData())) {
+            data->removeMark(mark);
+        }
+        block = block.next();
+    }
+    removeMarkFromMarksCache(mark);
+    mark->setMarkableInterface(0);
+}
+
+void DocumentMarker::updateMark(ITextMark *mark)
+{
+    Q_UNUSED(mark)
+    BaseTextDocumentLayout *documentLayout =
+        qobject_cast<BaseTextDocumentLayout*>(document->documentLayout());
+    QTC_ASSERT(documentLayout, return);
+    documentLayout->requestUpdate();
+}
+
+} // namespace Internal
+} // namespace TextEditor
+
 CodeFormatterData::~CodeFormatterData()
 {
 }
@@ -43,6 +175,10 @@ TextBlockUserData::~TextBlockUserData()
     TextMarks marks = m_marks;
     m_marks.clear();
     foreach (ITextMark *mrk, marks) {
+        TextEditor::Internal::DocumentMarker *documentMarker
+                = static_cast<TextEditor::Internal::DocumentMarker *>(mrk->markableInterface());
+        documentMarker->removeMarkFromMarksCache(mrk);
+        mrk->setMarkableInterface(0);
         mrk->removedFromEditor();
     }
 
@@ -190,10 +326,10 @@ TextBlockUserData::MatchType TextBlockUserData::checkClosedParenthesis(QTextCurs
             cursor->clearSelection();
             cursor->setPosition(openParenParag.position() + openParen.pos, QTextCursor::KeepAnchor);
 
-            if ((c == '}' && openParen.chr != '{')    ||
-                 (c == ')' && openParen.chr != '(')   ||
-                 (c == ']' && openParen.chr != '[')   ||
-                 (c == '-' && openParen.chr != '+'))
+            if ((c == QLatin1Char('}') && openParen.chr != QLatin1Char('{'))    ||
+                 (c == QLatin1Char(')') && openParen.chr != QLatin1Char('('))   ||
+                 (c == QLatin1Char(']') && openParen.chr != QLatin1Char('['))   ||
+                 (c == QLatin1Char('-') && openParen.chr != QLatin1Char('+')))
                 return Mismatch;
 
             return Match;
@@ -201,7 +337,7 @@ TextBlockUserData::MatchType TextBlockUserData::checkClosedParenthesis(QTextCurs
     }
 }
 
-bool TextBlockUserData::findPreviousOpenParenthesis(QTextCursor *cursor, bool select)
+bool TextBlockUserData::findPreviousOpenParenthesis(QTextCursor *cursor, bool select, bool onlyInCurrentBlock)
 {
     QTextBlock block = cursor->block();
     int position = cursor->position();
@@ -224,6 +360,8 @@ bool TextBlockUserData::findPreviousOpenParenthesis(QTextCursor *cursor, bool se
                 }
             }
         }
+        if (onlyInCurrentBlock)
+            return false;
         block = block.previous();
     }
     return false;
@@ -389,15 +527,19 @@ void TextBlockUserData::addMark(ITextMark *mark)
 
 
 BaseTextDocumentLayout::BaseTextDocumentLayout(QTextDocument *doc)
-    :QPlainTextDocumentLayout(doc) {
-    lastSaveRevision = 0;
-    hasMarks = 0;
-    maxMarkWidthFactor = 1.0;
-    m_requiredWidth = 0;
+    : QPlainTextDocumentLayout(doc),
+      lastSaveRevision(0),
+      hasMarks(false),
+      maxMarkWidthFactor(1.0),
+      m_requiredWidth(0),
+      m_documentMarker(new Internal::DocumentMarker(doc))
+{
+
 }
 
 BaseTextDocumentLayout::~BaseTextDocumentLayout()
 {
+    documentClosing();
 }
 
 void BaseTextDocumentLayout::setParentheses(const QTextBlock &block, const Parentheses &parentheses)
@@ -535,6 +677,11 @@ void BaseTextDocumentLayout::setFolded(const QTextBlock &block, bool folded)
     }
 }
 
+ITextMarkable *BaseTextDocumentLayout::markableInterface()
+{
+    return m_documentMarker;
+}
+
 void BaseTextDocumentLayout::doFoldOrUnfold(const QTextBlock& block, bool unfold)
 {
     if (!canFold(block))
@@ -576,6 +723,60 @@ QSizeF BaseTextDocumentLayout::documentSize() const
     return size;
 }
 
+TextMarks BaseTextDocumentLayout::documentClosing()
+{
+    TextMarks marks;
+    QTextBlock block = document()->begin();
+    while (block.isValid()) {
+        if (TextBlockUserData *data = static_cast<TextBlockUserData *>(block.userData()))
+            marks.append(data->documentClosing());
+        block = block.next();
+    }
+    return marks;
+}
+
+void BaseTextDocumentLayout::documentReloaded(TextMarks marks)
+{
+    foreach (ITextMark *mark, marks) {
+        int blockNumber = mark->lineNumber() - 1;
+        QTextBlock block = document()->findBlockByNumber(blockNumber);
+        if (block.isValid()) {
+            TextBlockUserData *userData = BaseTextDocumentLayout::userData(block);
+            userData->addMark(mark);
+            mark->setMarkableInterface(m_documentMarker);
+            mark->updateBlock(block);
+        } else {
+            TextEditor::Internal::DocumentMarker *documentMarker
+                    = static_cast<TextEditor::Internal::DocumentMarker *>(m_documentMarker);
+            documentMarker->removeMarkFromMarksCache(mark);
+            mark->removedFromEditor();
+        }
+    }
+    requestUpdate();
+}
+
+void BaseTextDocumentLayout::updateMarksLineNumber()
+{
+    // Note: the breakpointmanger deletes breakpoint marks and readds them
+    // if it doesn't agree with our updating
+    QTextBlock block = document()->begin();
+    int blockNumber = 0;
+    while (block.isValid()) {
+        if (const TextBlockUserData *userData = testUserData(block))
+            foreach (ITextMark *mrk, userData->marks())
+                mrk->updateLineNumber(blockNumber + 1);
+        block = block.next();
+        ++blockNumber;
+    }
+}
+
+void BaseTextDocumentLayout::updateMarksBlock(const QTextBlock &block)
+{
+    if (const TextBlockUserData *userData = testUserData(block))
+        foreach (ITextMark *mrk, userData->marks())
+            mrk->updateBlock(block);
+}
+
 BaseTextDocumentLayout::FoldValidator::FoldValidator()
     : m_layout(0)
     , m_requestDocUpdate(false)
@@ -641,3 +842,5 @@ void BaseTextDocumentLayout::FoldValidator::finalize()
         m_layout->emitDocumentSizeChanged();
     }
 }
+
+#include "basetextdocumentlayout.moc"
